@@ -52,6 +52,7 @@ class Todo:
     ]
 
     DO_NOT_TRACK = ('last_modified_date', 'sequence', 'completed_date', 'dtstamp', 'created_date')
+    DO_NOT_LOAD = ('uid', 'sequence', 'created_date', 'dtstamp', 'last_modified_date', 'hide_subtasks')
 
     UTC_DATE_FIELDS = ('created_date', 'completed_date', 'dtstamp', 'last_modified_date')
     LOCAL_DATE_FIELDS = ('start_date', 'end_date')
@@ -78,7 +79,7 @@ class Todo:
     )
 
     @classmethod
-    def load_from_vtodo(cls, vtodo):
+    def load_from_vtodo(cls, vtodo, load_all=True):
         kwargs = {}
 
         def _sanitize(val):
@@ -101,6 +102,8 @@ class Todo:
             return dt.astimezone(pytz.timezone(settings.TIMEZONE))
 
         for field_model, field_vtodo in cls.VTODO_MAPPING:
+            if not load_all and field_model in cls.DO_NOT_LOAD:
+                continue
             try:
                 val = vtodo[field_vtodo]
             except KeyError:
@@ -163,7 +166,7 @@ class Todo:
 
         if self.vtodo is None:
             self.vtodo = icalendar.Todo()
-            self.update_vtodo()
+        self.update_vtodo()
 
         def on_complete():
             if self.completed_date is not None and not self.completed:
@@ -231,7 +234,7 @@ class Todo:
         _now = self.now(aware=False)
 
         def _time_or_date_display(dt):
-            if not self.all_day and abs(_now - dt) < datetime.timedelta(days=180):
+            if abs(_now - dt) < datetime.timedelta(days=30):
                 return humanize.naturaltime(dt)
             else:
                 return humanize.naturaldate(dt)
@@ -257,12 +260,15 @@ class Todo:
 
         _start = _convert_datetime(self.start_date)
         _end = _convert_datetime(self.end_date)
-        if _start is not None and _end is None:
-            if _start > _now:
-                return 'Starts {}'.format(_time_or_date_display(_start))
-            else:
-                return ''
-        else:
+        if _start is not None and _start > _now:
+            return 'Starts {}'.format(_time_or_date_display(_start))
+        elif _end is not None:
+            if self.all_day and abs(_now - _end) < datetime.timedelta(days=1):
+                _end = datetime.datetime.combine(
+                    date=_end.date(),
+                    time=datetime.datetime.strptime(settings.ALL_DAY_DUE_TIME, '%H:%M').time(),
+                )
+
             if _end < _now:
                 return colour_text(
                     '<b>Ended {}</b>'.format(_time_or_date_display(_end)),
@@ -273,6 +279,7 @@ class Todo:
                     'Ends {}'.format(_time_or_date_display(_end)),
                     'orange',
                 )
+        return ''
 
     @property
     def sort_value(self):
@@ -353,21 +360,9 @@ class TodoEditor(Gtk.Grid):
         self.description_view.get_buffer().set_text(_data['description'])
         self.update_datepicker_labels()
 
-    def _get_label(self, widget):
-        date = widget.get_selected_date()
-        if date is None:
-            return 'Unset'
-        else:
-            time = '{hour}:{minute:02}'.format(hour=date.hour, minute=date.minute)
-            day = '{year}/{month:02}/{day:02}'.format(year=date.year, month=date.month, day=date.day)
-            if widget.get_date_only():
-                return day
-            else:
-                return '{day} {time}'.format(day=day, time=time)
-
     def update_datepicker_labels(self):
-        self.startdate_button.set_label(self._get_label(self.startdate_picker))
-        self.enddate_button.set_label(self._get_label(self.enddate_picker))
+        self.startdate_button.set_label(str(self.startdate_picker))
+        self.enddate_button.set_label(str(self.enddate_picker))
 
     def _create_widgets(self):
         self.summary_label = Gtk.Entry()
@@ -385,7 +380,7 @@ class TodoEditor(Gtk.Grid):
         self.startdate_picker = DateTimePickerWidget()
         self.startdate_popover.add(self.startdate_picker)
 
-        self.startdate_button = Gtk.Button(label=self._get_label(self.startdate_picker))
+        self.startdate_button = Gtk.Button(label=str(self.startdate_picker))
 
         def on_clicked_startdate(obj):
             self.startdate_picker.set_selected_date(self.data['start_date'])
@@ -420,7 +415,7 @@ class TodoEditor(Gtk.Grid):
         self.enddate_picker = DateTimePickerWidget()
         self.enddate_popover.add(self.enddate_picker)
 
-        self.enddate_button = Gtk.Button(label=self._get_label(self.enddate_picker))
+        self.enddate_button = Gtk.Button(label=str(self.enddate_picker))
 
         def on_clicked_enddate(obj):
             self.enddate_picker.set_selected_date(self.data['end_date'])
@@ -614,8 +609,8 @@ class TodoListWindow(Gtk.Grid):
         self.tree_view.connect('row-expanded', self.tree_view_row_visibility_changed, False)
         self.tree_view.get_selection().connect('changed', self.tree_selection_changed)
         self.tree_view.set_enable_tree_lines(True)
-        self.tree_view.set_enable_search(False)
-        self.tree_view.set_search_column(-1)
+        self.tree_view.set_enable_search(True)
+        self.tree_view.set_search_column(0)
         self.tree_view.set_tooltip_column(0)
         self.todo_scrollable_view = Gtk.ScrolledWindow(vexpand=True)
         self.todo_scrollable_view.set_propagate_natural_width(True)
@@ -655,6 +650,15 @@ class TodoListWindow(Gtk.Grid):
         button_delete.connect('clicked', on_todo_delete)
         self.popover_grid.add(button_delete)
 
+        button_clone_subtask = Gtk.Button(label='Clone')
+        button_clone_subtask.set_image(Gtk.Image.new_from_icon_name('edit-copy-symbolic', 4))
+        button_clone_subtask.set_always_show_image(True)
+
+        def on_clone_subtask(obj):
+            self.clone_todo(attached_uid=self.popover._attached_uid)
+        button_clone_subtask.connect('clicked', on_clone_subtask)
+        self.popover_grid.add(button_clone_subtask)
+
         button_add_subtask = Gtk.Button(label='Add subtask')
         button_add_subtask.set_image(Gtk.Image.new_from_icon_name('list-add-symbolic', 4))
         button_add_subtask.set_always_show_image(True)
@@ -688,7 +692,10 @@ class TodoListWindow(Gtk.Grid):
             adjacency_list = defaultdict(list)
 
             for vtodo in server.todolist[self._current_calendar]:
-                self.data[str(vtodo['UID'])] = Todo.load_from_vtodo(vtodo)
+                todo = Todo.load_from_vtodo(vtodo)
+                if todo.completed and int(settings.HIDE_COMPLETED):
+                    continue
+                self.data[str(vtodo['UID'])] = todo
 
             for todo in self.data.values():
                 parent = todo['related_to']
@@ -730,12 +737,15 @@ class TodoListWindow(Gtk.Grid):
             ],
         )
 
-    def new_todo(self, parent_uid=None):
+    def new_todo(self, parent_uid=None, new_todo=None):
         if self._current_calendar is None:
             return
 
+        if new_todo is None:
+            new_todo = Todo(related_to=parent_uid)
+
         parent_it = self.todo_uid_to_iter.get(parent_uid, None)
-        new_todo = self.data[new_todo.uid] = Todo(related_to=parent_uid)
+        self.data[new_todo.uid] = new_todo
         todo_it = self.todo_uid_to_iter[new_todo.uid] = self.attach_todo(parent_it, new_todo.uid)
 
         server.update_todo(new_todo.vtodo, self._current_calendar)
@@ -745,6 +755,15 @@ class TodoListWindow(Gtk.Grid):
         self.tree_view.expand_to_path(path)
         self.tree_view.set_cursor(path, None, False)
 
+    def clone_todo(self, attached_uid=None):
+        try:
+            data = self.data[attached_uid]
+        except KeyError:
+            return
+
+        cloned_todo = Todo.load_from_vtodo(icalendar.Todo.from_ical(data.vtodo.to_ical()), load_all=False)
+        self.new_todo(cloned_todo.related_to, new_todo=cloned_todo)
+
     def connect_todo(self, todo):
         path = self.store.get_path(self.todo_uid_to_iter[todo.uid])
         for field in ('progress', 'status', 'summary'):
